@@ -1,13 +1,13 @@
-pub mod display;
+pub mod cli_display;
+pub mod filter;
 pub mod msg;
 pub mod setup;
+pub mod templates;
+pub mod views;
 
-use mapm::problem::Filter;
 use mapm::problem::FilterAction;
 use mapm::problem::Views::*;
 use mapm::problem::*;
-use mapm::result::MapmErr::*;
-use mapm::result::MapmResult::*;
 
 use std::env;
 use std::fs;
@@ -17,7 +17,6 @@ use std::process::Command;
 use exitcode;
 use quit;
 
-use ansi_term::Colour::*;
 use colour::*;
 
 fn build_contest(contest_result: mapm::contest::ContestResult, template_dir: &Path) {
@@ -64,117 +63,13 @@ fn build_contest(contest_result: mapm::contest::ContestResult, template_dir: &Pa
     }
 }
 
-fn get_problems_string(profile: &str, problem_names: &Vec<String>, views: Views) -> String {
-    let home = dirs::home_dir().unwrap();
-    let mut problem_display = String::new();
-    let mut problem_index = 0;
-    for problem_name in problem_names {
-        problem_index += 1;
-        if problem_index > 1 {
-            problem_display.push_str("\n\n");
-        }
-        let problem_result = fetch_problem(
-            problem_name,
-            &Path::new(&home)
-                .join(".mapm")
-                .join("problems")
-                .join(profile),
-        );
-        match problem_result {
-            Success(problem) => {
-                let filtered_problem = problem.filter_keys(views.clone());
-                problem_display.push_str(
-                    &Green
-                        .bold()
-                        .paint(["-- ", problem_name, " --"].concat())
-                        .to_string(),
-                );
-                for (key, val) in &filtered_problem.0 {
-                    problem_display.push_str("\n");
-                    problem_display.push_str(&Blue.paint(key).to_string());
-                    problem_display.push_str(": ");
-                    let mut idx = 0;
-                    for string in val.split("\n") {
-                        idx += 1;
-                        if idx == 1 {
-                            problem_display.push_str(&string);
-                        } else {
-                            problem_display.push_str("\n  ");
-                            problem_display.push_str(&str::repeat(" ", key.chars().count()));
-                            problem_display.push_str(&string);
-                        }
-                    }
-                }
-                match &filtered_problem.1 {
-                    Some(solutions) => {
-                        problem_display.push_str("\n");
-                        problem_display.push_str(&Blue.bold().paint("solutions").to_string());
-                        problem_display.push_str(": ");
-                        if solutions.len() == 0 {
-                            problem_display.push_str(&Red.bold().paint("empty").to_string());
-                        } else {
-                            let mut index: u32 = 0;
-                            for solution in solutions {
-                                index += 1;
-                                problem_display.push_str("\n  ");
-                                problem_display.push_str(
-                                    &Red.paint([&index.to_string(), "."].concat()).to_string(),
-                                );
-                                let mut key_index = 0;
-                                for (key, val) in solution {
-                                    key_index += 1;
-                                    if key_index == 1 {
-                                        problem_display.push_str(" ");
-                                    } else {
-                                        problem_display.push_str("\n");
-                                        problem_display.push_str(&str::repeat(" ", 5));
-                                    }
-                                    problem_display.push_str(&Blue.paint(key).to_string());
-                                    problem_display.push_str(": ");
-                                    let mut idx = 0;
-                                    for string in val.split("\n") {
-                                        idx += 1;
-                                        if idx == 1 {
-                                            problem_display.push_str(&string);
-                                        } else {
-                                            problem_display.push_str("\n");
-                                            problem_display.push_str(&str::repeat(
-                                                " ",
-                                                index.to_string().len() + key.chars().count() + 6,
-                                            ));
-                                            problem_display.push_str(&string);
-                                        }
-                                    }
-                                }
-                            }
-                        }
-                    }
-                    None => {
-                        problem_display.push_str("\n");
-                    }
-                }
-            }
-            Fail(err) => match err {
-                ProblemErr(msg) => {
-                    e_yellow_ln!("{}", msg);
-                }
-                _ => {
-                    e_magenta_ln!("{}", msg::VIEW_NOT_PROBLEMERR);
-                    quit::with_code(exitcode::SOFTWARE);
-                }
-            },
-        }
-    }
-    return problem_display;
-}
-
 #[quit::main]
 fn main() {
-    let home = dirs::home_dir().unwrap();
+    let config = dirs::config_dir().unwrap();
     let profile: String;
-    let profile_config_path = Path::new(&home).join(".mapm").join("profile");
+    let profile_config_path = Path::new(&config).join("mapm").join("profile");
     if profile_config_path.is_file() {
-        match fs::read_to_string(Path::new(&home).join(".mapm").join("profile")) {
+        match fs::read_to_string(Path::new(&config).join("mapm").join("profile")) {
             Ok(profile_str) => {
                 profile = profile_str;
             }
@@ -184,11 +79,11 @@ fn main() {
             }
         }
     } else {
-        setup::setup();
+        setup::setup_prompt();
         return;
     }
-    let problem_dir = Path::new(&home)
-        .join(".mapm")
+    let problem_dir = Path::new(&config)
+        .join("mapm")
         .join("problems")
         .join(&profile);
     if !problem_dir.is_dir() {
@@ -202,10 +97,13 @@ fn main() {
         );
     }
 
-    let template_dir = Path::new(&home).join(".mapm").join("templates");
+    let template_dir = Path::new(&config).join("mapm").join("templates");
 
     if !template_dir.is_dir() {
-        fs::create_dir_all(&template_dir).expect("Could not create directory `~/.mapm/templates`");
+        fs::create_dir_all(&template_dir).expect(&format!(
+            "Could not create directory `{}`",
+            &template_dir.to_str().unwrap()
+        ));
     }
 
     let mut actions: Vec<(String, Vec<String>)> = Vec::new();
@@ -261,8 +159,10 @@ fn main() {
                     } else {
                         views = Hide(Vec::new());
                     }
-                    let problem_display = get_problems_string(&profile, &actions[1].1, views);
-                    display::display(&problem_display);
+                    let problem_display = cli_display::problems_to_string(
+                        views::problem_names_to_problems(&profile, &actions[1].1, views),
+                    );
+                    cli_display::display(&problem_display);
                 }
                 "find" => {
                     if actions[1].1.iter().any(|s| s == "help") {
@@ -271,147 +171,68 @@ fn main() {
                     }
                     let views: Views;
                     if actions.len() > 3 {
-                        e_red_ln!("{} (mapm view --help for help)", msg::TOO_MANY_ARGS);
+                        e_red_ln!("{} (mapm find --help for help)", msg::TOO_MANY_ARGS);
                         quit::with_code(exitcode::USAGE);
                     } else if actions.len() == 3 {
-                        match actions[2].0.as_str() {
-                            "hide" => {
-                                let mut views_vec: Vec<String> = Vec::new();
-                                for view in &actions[2].1 {
-                                    views_vec.push(String::from(view));
-                                }
-                                views = Hide(views_vec);
-                            }
-                            "show" => {
-                                let mut views_vec: Vec<String> = Vec::new();
-                                for view in &actions[2].1 {
-                                    views_vec.push(String::from(view));
-                                }
-                                views = Show(views_vec);
-                            }
-                            _ => {
-                                e_red_ln!("{}", msg::INVALID_SUBCMD);
-                                quit::with_code(exitcode::USAGE);
-                            }
-                        }
+                        views = views::parse_views(&actions[2]);
                     } else {
                         views = Hide(Vec::new());
                     }
-                    let mut filters: Vec<FilterAction> = Vec::new();
-
-                    fn parse_filter(arg: &str) -> Filter {
-                        match arg.find("<=") {
-                            Some(idx) => match arg[idx + 2..].parse::<u32>() {
-                                Ok(val) => {
-                                    return Filter::Le {
-                                        key: String::from(&arg[0..idx]),
-                                        val,
-                                    };
-                                }
-                                Err(_) => {
-                                    e_red_ln!("Filter key `{}` was passed with non-integer value `{}`, despite non-equality comparison operator `<=`", &arg[0..idx], &arg[idx+2..]);
-                                    quit::with_code(exitcode::USAGE);
-                                }
-                            },
-                            None => match arg.find(">=") {
-                                Some(idx) => match arg[idx + 2..].parse::<u32>() {
-                                    Ok(val) => {
-                                        return Filter::Ge {
-                                            key: String::from(&arg[0..idx]),
-                                            val,
-                                        };
-                                    }
-                                    Err(_) => {
-                                        e_red_ln!("Filter key `{}` was passed with non-integer value `{}`, despite non-equality comparison operator `>=`", &arg[0..idx], &arg[idx+2..]);
-                                        quit::with_code(exitcode::USAGE);
-                                    }
-                                },
-
-                                None => match arg.find("=") {
-                                    Some(idx) => {
-                                        return Filter::Eq {
-                                            key: String::from(&arg[0..idx]),
-                                            val: String::from(&arg[idx + 1..]),
-                                        }
-                                    }
-                                    None => match arg.find("<") {
-                                        Some(idx) => match arg[idx + 1..].parse::<u32>() {
-                                            Ok(val) => {
-                                                return Filter::Le {
-                                                    key: String::from(&arg[0..idx]),
-                                                    val,
-                                                };
-                                            }
-                                            Err(_) => {
-                                                e_red_ln!("Filter key `{}` was passed with non-integer value `{}`, despite non-equality comparison operator `<`", &arg[0..idx], &arg[idx+1..]);
-                                                quit::with_code(exitcode::USAGE);
-                                            }
-                                        },
-                                        None => match arg.find(">") {
-                                            Some(idx) => match arg[idx + 1..].parse::<u32>() {
-                                                Ok(val) => {
-                                                    return Filter::Le {
-                                                        key: String::from(&arg[0..idx]),
-                                                        val,
-                                                    };
-                                                }
-                                                Err(_) => {
-                                                    e_red_ln!("Filter key `{}` was passed with non-integer value `{}`, despite non-equality comparison operator `>`", &arg[0..idx], &arg[idx+1..]);
-                                                    quit::with_code(exitcode::USAGE);
-                                                }
-                                            },
-                                            None => {
-                                                return Filter::Exists {
-                                                    key: String::from(arg),
-                                                }
-                                            }
-                                        },
-                                    },
-                                },
-                            },
-                        }
+                    let filters: Vec<FilterAction> = filter::parse_filters(&actions[1].1);
+                    let problem_names = filter::filtered_names(&filters, &problem_dir);
+                    let problem_display = cli_display::problems_to_string(
+                        views::problem_names_to_problems(&profile, &problem_names, views),
+                    );
+                    cli_display::display(&problem_display);
+                }
+                "preview-all" => {
+                    if actions[1].1.iter().any(|s| s == "help") {
+                        println!("{}", msg::HELP_FIND);
+                        quit::with_code(exitcode::OK);
                     }
-
-                    for arg in &actions[1].1 {
-                        if &arg[0..1] == "!" {
-                            filters.push(FilterAction::Negative(parse_filter(&arg[1..])));
-                        } else {
-                            filters.push(FilterAction::Positive(parse_filter(arg)));
-                        }
+                    let views: Views;
+                    if actions.len() > 3 {
+                        e_red_ln!("{} (mapm find --help for help)", msg::TOO_MANY_ARGS);
+                        quit::with_code(exitcode::USAGE);
+                    } else if actions.len() == 3 {
+                        views = views::parse_views(&actions[2]);
+                    } else {
+                        views = Hide(Vec::new());
                     }
-                    let mut problem_names: Vec<String> = Vec::new();
-                    for file in fs::read_dir(problem_dir).unwrap() {
-                        let problem_yaml =
-                            fs::read_to_string(file.as_ref().unwrap().path()).unwrap();
-                        let problem_res = mapm::problem::parse_problem_yaml(&problem_yaml);
-                        match problem_res {
-                            Success(problem) => match problem.filter(filters.clone()) {
-                                Some(_) => {
-                                    problem_names.push(String::from(
-                                        file.as_ref()
-                                            .unwrap()
-                                            .path()
-                                            .file_stem()
-                                            .unwrap()
-                                            .to_str()
-                                            .unwrap(),
-                                    ));
-                                }
-                                None => {}
-                            },
-                            Fail(err) => match err {
-                                ProblemErr(msg) => {
-                                    e_yellow_ln!("{}", msg);
-                                }
-                                _ => {
-                                    e_magenta_ln!("{}", msg::FIND_NOT_PROBLEMERR);
-                                    quit::with_code(exitcode::SOFTWARE);
-                                }
-                            },
-                        }
+                    let filters: Vec<FilterAction> = filter::parse_filters(&actions[1].1);
+                    let problem_names = filter::filtered_names(&filters, &problem_dir);
+                    let problems =
+                        views::problem_names_to_problems(&profile, &problem_names, views);
+                    let problems_tex = templates::preview_all_tex(problems);
+                    let cwd = env::current_dir().expect("Could not get current directory");
+                    let cache_dir = dirs::cache_dir().expect("Could not find cache directory");
+                    let preview_dir = cache_dir.join("mapm").join("preview-all");
+                    if !preview_dir.is_dir() {
+                        fs::create_dir_all(&preview_dir).expect("Could not create directory");
                     }
-                    let problem_display = get_problems_string(&profile, &problem_names, views);
-                    display::display(&problem_display);
+                    fs::write(preview_dir.join("preview-all.tex"), &problems_tex).expect(&format!(
+                        "Could not write to `{}`",
+                        preview_dir.join("preview-all.tex").to_str().unwrap()
+                    ));
+                    env::set_current_dir(&preview_dir).expect("Could not set working directory");
+                    let latexmk = Command::new("latexmk")
+                        .args(["-pdf", "preview-all.tex"])
+                        .output()
+                        .expect("Failed to execute `latexmk`; make sure you have it installed");
+                    if latexmk.status.success() {
+                        println!(
+                            "`latexmk` on `{}` succeeded",
+                            preview_dir.join("preview-all.tex").to_str().unwrap()
+                        );
+                    } else {
+                        e_red_ln!(
+                            "`latexmk` on `{}` failed",
+                            preview_dir.join("preview-all.tex").to_str().unwrap()
+                        );
+                    }
+                    open::that(preview_dir.join("preview-all.pdf"))
+                        .expect("Could not open `preview-all.pdf`");
+                    env::set_current_dir(&cwd).expect("Could not set working directory");
                 }
                 "edit" => {
                     if actions[1].1.iter().any(|s| s == "help") {
@@ -472,16 +293,21 @@ fn main() {
                                 }
                                 println!("{}", &profile);
                             }
-                            "set" => {
-                                if actions.len() > 3 {
+                            "set" => match actions.len() {
+                                3 => {
+                                    setup::setup_prompt();
+                                }
+                                4 => {
+                                    setup::setup(&actions.get(3).unwrap().0);
+                                }
+                                _ => {
                                     e_red_ln!(
                                         "{} (mapm profile --help for help)",
                                         msg::TOO_MANY_ARGS
                                     );
                                     quit::with_code(exitcode::USAGE);
                                 }
-                                setup::setup();
-                            }
+                            },
                             _ => {
                                 e_red_ln!("{}", msg::INVALID_PROFILE_SUBCMD);
                                 quit::with_code(exitcode::USAGE);
@@ -506,14 +332,7 @@ fn main() {
                 "preview" => {
                     for action in &actions[2..] {
                         let problem_name = &action.0;
-                        let contest_yml = &[
-                            "template: preview
-vars: {}
-problems:
-  - ",
-                            problem_name,
-                        ]
-                        .concat();
+                        let contest_yml = &templates::preview_template(problem_name);
                         let contest_result = mapm::contest::parse_contest_yaml(
                             contest_yml,
                             &problem_dir,
